package com.zhiyun.project.project.service;

import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.function.BiFunction;

/**
 * mybatis 批量处理
 *
 * @author yxk
 * @since 2024/8/29 11:01
 */
@Slf4j
@Component
public class MybatisBatchService {

    /**
     * 每次处理1000条
     */
    private static final int BATCH_SIZE = 1000;

    @Resource
    private SqlSessionFactory sqlSessionFactory;

    /**
     * 对数据列表进行批量更新或插入操作。
     *
     * @param data        需要被处理的数据列表
     * @param mapperClass MyBatis 的 Mapper 类
     * @param function    用于定义如何处理每一个数据项的函数
     */
    public <T, U, R> void batchUpdateOrInsert(List<T> data, Class<U> mapperClass, BiFunction<T, U, R> function) {
        if (data == null || data.isEmpty() || mapperClass == null) {
            return;
        }
        SqlSession batchSqlSession = sqlSessionFactory.openSession(ExecutorType.BATCH);
        try {
            U mapper = batchSqlSession.getMapper(mapperClass);
            for (int i = 0; i < data.size(); i++) {
                function.apply(data.get(i), mapper);
                // 每处理 BATCH_SIZE 条数据，或者已经处理到数据列表的最后一项，就执行一次刷新
                if (i != 0 && (i % BATCH_SIZE == 0 || i == data.size() - 1)) {
                    batchSqlSession.flushStatements();
                }
            }
            batchSqlSession.flushStatements();
            batchSqlSession.commit();
        } catch (Exception e) {
            batchSqlSession.rollback();
            throw e;
        } finally {
            batchSqlSession.close();
        }
    }

    /**
     * 对数据列表进行批量更新或插入操作。
     */
    public <T, U, R> void batchInsert(List<T> data, Class<U> mapperClass, BiFunction<List<T>, U, R> function) {
        if (data == null || data.isEmpty() || mapperClass == null) {
            return;
        }
        SqlSession batchSqlSession = sqlSessionFactory.openSession(ExecutorType.BATCH);
        try {
            U mapper = batchSqlSession.getMapper(mapperClass);
            List<List<T>> partition = Lists.partition(data, 50);
            for (List<T> list : partition) {
                function.apply(list, mapper);
            }
            batchSqlSession.flushStatements();
            batchSqlSession.commit();
        } catch (Exception e) {
            batchSqlSession.rollback();
        } finally {
            batchSqlSession.close();
        }
    }

}
